(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('prop-types')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react', 'prop-types'], factory) :
	(factory((global.statty = {}),global.React,global.PropTypes));
}(this, (function (exports,react,PropTypes) { 'use strict';

PropTypes = PropTypes && PropTypes.hasOwnProperty('default') ? PropTypes['default'] : PropTypes;

var immutable = extend;

var hasOwnProperty = Object.prototype.hasOwnProperty;

function extend() {
    var arguments$1 = arguments;

    var target = {};

    for (var i = 0; i < arguments.length; i++) {
        var source = arguments$1[i];

        for (var key in source) {
            if (hasOwnProperty.call(source, key)) {
                target[key] = source[key];
            }
        }
    }

    return target
}

var State = (function (Component$$1) {
  function State (props, context) {
    Component$$1.call(this, props, context);
    this.broadcast = context.__statty__.broadcast;
    this.gs = this.broadcast.getState;
    this.state = props.state ? props.state : this.gs();
    this.update = this.update.bind(this);
    this.setState = this.setState.bind(this);
  }

  if ( Component$$1 ) State.__proto__ = Component$$1;
  State.prototype = Object.create( Component$$1 && Component$$1.prototype );
  State.prototype.constructor = State;

  State.prototype.debug = function debug (oldState, newState) {
    try {
      console.group('%c action%c name', 'color: #AAAAAA', 'color: #001B44');
    } catch (e) {
      console.log('action');
    }
    console.log('%c old state', 'color: #E7040F', oldState);
    console.log('%c action', 'color: #111111');
    console.log('%c new state', 'color: #19A974', newState);
    try {
      console.groupEnd();
    } catch (e) {
      console.log('== end ==');
    }
  };

  State.prototype.update = function update (fn) {
    if (this.props.state) {
      this.setState(fn);
    } else {
      var oldState = this.gs();
      var newState = immutable(oldState, fn(oldState));
      this.context.__statty__.debug && this.debug(oldState, newState);
      this.broadcast.setState(newState);
    }
  };

  State.prototype.componentDidMount = function componentDidMount () {
    if (!this.props.state) {
      this.subscriptionId = this.broadcast.subscribe(this.setState);
    }
  };

  State.prototype.componentWillUnmount = function componentWillUnmount () {
    this.subscriptionId && this.broadcast.unsubscribe(this.subscriptionId);
  };

  State.prototype.render = function render () {
    if (!this.props.render) { return null }
    var mapped = this.props.select(this.props.state ? this.state : this.gs());
    return this.props.render(mapped, this.update) || null
  };

  return State;
}(react.Component));

State.defaultProps = {
  select: function (s) { return s; }
};

State.contextTypes = {
  __statty__: PropTypes.object.isRequired
};

State.propTypes = {
  update: PropTypes.func,
  state: PropTypes.object,
  render: PropTypes.func
};

function createBroadcast$1 (initialState) {
  var listeners = {};
  var id = 1;
  var _state = initialState;

  function getState () {
    return _state
  }

  function setState (state) {
    _state = state;
    var keys = Object.keys(listeners);
    var i = 0;
    var len = keys.length;
    for (; i < len; i++) {
      // if a listener gets unsubscribed during setState we just skip it
      if (listeners[keys[i]]) { listeners[keys[i]](state); }
    }
  }

  // subscribe to changes and return the subscriptionId
  function subscribe (listener) {
    if (typeof listener !== 'function') {
      throw new Error('listener must be a function.')
    }
    var currentId = id;
    listeners[currentId] = listener;
    id += 1;
    return currentId
  }

  // remove subscription by removing the listener function
  function unsubscribe (id) {
    listeners[id] = undefined;
  }

  return { getState: getState, setState: setState, subscribe: subscribe, unsubscribe: unsubscribe }
}

var Provider = (function (Component$$1) {
  function Provider (props, context) {
    Component$$1.call(this, props, context);
    this.broadcast = createBroadcast$1(props.state);
    this.debug = createBroadcast$1(props.debug);
  }

  if ( Component$$1 ) Provider.__proto__ = Component$$1;
  Provider.prototype = Object.create( Component$$1 && Component$$1.prototype );
  Provider.prototype.constructor = Provider;

  Provider.prototype.getChildContext = function getChildContext () {
    return {
      __statty__: {
        broadcast: this.broadcast,
        debug: this.debug
      }
    }
  };

  Provider.prototype.render = function render () {
    var c = this.props.children;
    return Array.isArray(c) ? c[0] : c
  };

  return Provider;
}(react.Component));

Provider.childContextTypes = {
  __statty__: PropTypes.object
};

Provider.propTypes = {
  debug: PropTypes.bool,
  state: PropTypes.object
};

exports.Provider = Provider;
exports.State = State;

Object.defineProperty(exports, '__esModule', { value: true });

})));
