import { Component } from 'react'
import PropTypes from 'prop-types'
import xtend from 'xtend'

class State extends Component {
  constructor (props, context) {
    super(props, context)
    this.broadcast = context.__statty__.broadcast
    this.gs = this.broadcast.getState
    this.state = props.state ? props.state : this.gs()
    this.update = this.update.bind(this)
    this.setState = this.setState.bind(this)
  }

  debug (oldState, newState) {
    try {
      console.group('%c action%c name', 'color: #AAAAAA', 'color: #001B44')
    } catch (e) {
      console.log('action')
    }
    console.log('%c old state', 'color: #E7040F', oldState)
    console.log('%c action', 'color: #111111')
    console.log('%c new state', 'color: #19A974', newState)
    try {
      console.groupEnd()
    } catch (e) {
      console.log('== end ==')
    }
  }

  update (fn) {
    if (this.props.state) {
      this.setState(fn)
    } else {
      const oldState = this.gs()
      const newState = xtend(oldState, fn(oldState))
      this.context.__statty__.debug && this.debug(oldState, newState)
      this.broadcast.setState(newState)
    }
  }

  componentDidMount () {
    if (!this.props.state) {
      this.subscriptionId = this.broadcast.subscribe(this.setState)
    }
  }

  componentWillUnmount () {
    this.subscriptionId && this.broadcast.unsubscribe(this.subscriptionId)
  }

  render () {
    if (!this.props.render) return null
    const mapped = this.props.select(this.props.state ? this.state : this.gs())
    return this.props.render(mapped, this.update) || null
  }
}

State.defaultProps = {
  select: s => s
}

State.contextTypes = {
  __statty__: PropTypes.object.isRequired
}

State.propTypes = {
  update: PropTypes.func,
  state: PropTypes.object,
  render: PropTypes.func
}

export default State
